--[[
-- media file
-- This file loads and controls all the sounds of the game.
-- * media.load() reads the sounds from the disk. It must be called before
--   the sounds or music are used
-- * media.music contains a source with the music
-- * media.sfx.* contains multisources (see lib/multisource.lua)
-- * media.cleanup liberates unused sounds.
-- * media.countInstances counts how many sound instances are there in the
--   system. This is used for debugging
]]

local media = {}

local function newSource(name,ext,path)
  local path = path or 'assets/sfx/' .. name .. (ext or '.wav')
  local source = love.audio.newSource(path,"static")
  return multisource.new(source)
end

media.newSource = newSource

function media.loadFile(path,name)
    local ext = name:sub(-4,-1)
        
    local s = name:sub(1,-5)
    media.sfx[s] = newSource(s,ext,path)
    media[s] = media.sfx[s]
end

function media.loadMusicFile(path,name,volume)
    local ext = name:sub(-4,-1)
        
    local s = name:sub(1,-5)
    media.loadMusic(s,nil,path,nil,volume)
end

local function getMusic()
    return "tmp.mp3"
end

math.random()
media.load = function()

  
  local files = love.filesystem.getDirectoryItems("assets/sfx")

  media.sfx = {}
  media.musics = {}
  media.volumes = {}
  
  for i,v in ipairs(files) do
    local name = "assets/sfx/"..v
    if love.filesystem.getInfo(name).type=="file" then
        local ext = v:sub(-4,-1)
        
        local s = v:sub(1,-5)
        media.sfx[s] = newSource(s,ext)
        media[s] = media.sfx[s]
    end
    
  end


  --media.sfx.player_propulsion:setLooping(true)
  media.sfx.boom = media.sfx.explosion
  
  function media:isStopped() return false end
  media.loadMusic("tmp",nil,nil,".mp3",.3)

end

media.loadMusic = function(source, play, path, ext, volume)
  if media.musics[source] then
    media.music = media.musics[source]
  else
    media.music = love.audio.newSource(path or "assets/sfx/"..(source or getMusic())..ext,"static")
  end
  
  media.music:setLooping(true)
  media.musics[source] = media.music
  media.volumes[source] = volume or .5
  
  if play then
    media.music.setVolume(media.music,media.volumes[source])
    media.music:play()
  end
  
  return media.music
end

media.cleanup = function()
  for _,sfx in pairs(media.sfx) do
    sfx:cleanup()
  end
end

media.countInstances = function()
  local count = 0
  for _,sfx in pairs(media.sfx) do
    count = count + sfx:countInstances()
  end
  return count
end


return media
