----------------------------------------------------------------------------
--------------------   Stick creator forked from gooi   --------------------
----------------------------------------------------------------------------

Joystick = class:extend("Joystick")
Joystick.unit = 25

Joystick.colors = {
  blue = {0.01, 0.46, 0.85, 1.0},
  green = {0.36, 0.72, 0.36, 1.0},
  cyan = {0.36, 0.75, 0.87, 1.0},
  orange = {0.94, 0.68, 0.31, 1.0},
  red = {0.85, 0.33, 0.31, 1.0},
  black = {0.0, 0.0, 0.0, 1.0},
  white = {1.0, 1.0, 1.0, 1.0},
  clearGray = {0.97, 0.97, 0.97, 1.0},
  darkGray = {0.16, 0.17, 0.17, 1.0},
  darkGrayAlpha = {0.16, 0.17, 0.17, 0.59},
}
Joystick.style = {
  bgColor = Joystick.colors.blue,
  fgColor = Joystick.colors.white, -- Foreground color
  tooltipFont = love.graphics.newFont(love.window.toPixels(11)), -- tooltips are smaller than the main font
  radius = 2, -- raw pixels
  innerRadius = 2, -- raw pixels
  showBorder = true, -- border for components
  borderColor = Joystick.colors.blue,
  borderWidth = love.window.toPixels(2), -- in pixels
  borderStyle = "smooth", -- or "smooth"
  font = love.graphics.newFont(love.window.toPixels(13)),
}
function Joystick:__init__(kwargs)
    local kwargs = kwargs or {}
    local defSize = Joystick.unit* 4
    self.style = {
  bgColor = self.colors.blue,
  fgColor = self.colors.white, -- Foreground color
  tooltipFont = love.graphics.newFont(love.window.toPixels(11)), -- tooltips are smaller than the main font
  radius = 2, -- raw pixels
  innerRadius = 2, -- raw pixels
  showBorder = true, -- border for components
  borderColor = self.colors.blue,
  borderWidth = love.window.toPixels(2), -- in pixels
  borderStyle = "smooth", -- or "smooth"
  font = love.graphics.newFont(love.window.toPixels(13)),
}

    self.x, self.y, self.w, self.h, self.sx, self.sy ,self.type = --gooi.checkBounds(
    
        kwargs.x or 10,
        kwargs.y or 10,
        kwargs.w or defSize,
        kwargs.h or defSize,
        kwargs.sx or 1,
        kwargs.sy or 1,
        "joystick"


    if kwargs.cx then
        self.x = kwargs.cx-self.w/2
    end
    if kwargs.cy then
        self.y = kwargs.cy-self.h/2
    end
    local _x,_y = self.x,self.y
    self._x, self._y = _x, _y
    
    self.smallerSide = self.h
    if self.w < self.h then
        self.smallerSide = self.w
    end

    -- Note that the sitck has x and y on the center.
    --s = component.new("joystick", x, y, kwargs.size or defSize, kwargs.size or defSize, kwargs   .group)
    --s = gooi.setStyleComp(s)
    self.radiusCorner = self.h / 2
    self.deadZone = kwargs.deadZone or 0 -- Given in percentage (0 to 1).
    if self.deadZone < 0  then self.deadZone = 0 end
    if self.deadZone > 1  then self.deadZone = 1 end
    self.stickPressed = false
    self.dx, self.dy = 0, 0
    self.spring = true
    self.sxImg, self.syImg = 1, 1
    self.digitalH, self.digitalV = "", ""
    self.touch = nil-- Stores the touch which is on this component.
    self.opaque = true-- If false, the component base will never be drawn.
    self.events = {p = nil, r = nil, m = nil}
    self:rebuild()
    self:setImage(kwargs.image)
    self:enable()
end

function Joystick:enable()
    self.enabled = true
    self.visible = true
end

function Joystick:disable()
    self.enabled = false
    self.visible = false
end

function Joystick:draw(fg)
    local l,m,n,o = love.graphics.getColor()
    local old = {l,m,n,o}
    local trash = fg and love.graphics.setColor(fg)
    self:drawStick()
    love.graphics.setColor(old[1], old[2], old[3], old[4])
end
    
function Joystick:rebuild()
    self.r = self.smallerSide / 2
    self.rStick = self.r / 2
    self.xStick = (self.x) + (self.r)
    self.yStick = (self.y) + (self.r)
    self._ry = self.yStick
    self._rx = self.xStick
    --self:generateBorder()
end

function Joystick:setImage(image)
    if image then
        if type(image) == "string" then
          image = love.graphics.newImage(image)
        end
        self.image = image
        self.image:setFilter("linear", "linear")
    end
    return self
end

function Joystick:noScaling()
    self.notS = true
    return self
end

function Joystick:anyPoint()
    self.anyP = true
    return self
end

function Joystick:setDigital(directions)-- 4 or 8
    if directions and
    directions ~= "4" and
    directions ~= "8" then

    end
    self.digital = directions or "8"
    return self
end

function Joystick:drawStick()
    local fg = self.style.fgColor
    if self.image then
        love.graphics.setColor(1, 1, 1, fg[4] or 1)
        local sx = self.rStick * 2 / self.image:getWidth()
        local sy = self.rStick * 2 / self.image:getHeight()
        local x, y = self.xStick, self.yStick
        if self.notS then
            sx, sy = 1, 1
            x, y = (self.xStick), (self.yStick)
        end

        if self.digital then
            x, y = self:computeDigital()
        end

        love.graphics.draw(self.image,
            math.floor(x), math.floor(y),
            0, sx, sy,
            self.image:getWidth() / 2,
            self.image:getHeight() / 2)
    else
        local x = self.xStick
        local y = self.yStick

        if self.digital then
            x, y = self:computeDigital()
        end
        love.graphics.setColor(9.607,90.607,0.607,76.196)
        --love.graphics.circle("fill",self.xStick,self.yStick,20)--20,20)
        love.graphics.circle("line", self._rx, self._ry, self.rStick*2)--self.xStick-(self.rStick*0),self._y-(self.rStick),self.rStick*2)--,circleRes*2) 
        love.graphics.setColor(255,255,255)
        love.graphics.circle("line", math.floor(x), math.floor(y), self.rStick, circleRes)
    end
end

function Joystick:computeDigital()
    -- horizontal directions:
    local xv = self:xValue()
    local yv = self:yValue()

    if self.digital == "8" then
        -- horizontal direction:
        if xv < -0.5 then
            self.digitalH = "l"
            x = self.x + self.rStick
        elseif xv > 0.5 then
            self.digitalH = "r"
            x = self.x + self.w - self.rStick
        else
            self.digitalH = ""
            x = self.x + self.w / 2
        end
        --vertical:
        if yv < -0.5 then
            self.digitalV = "t"
            y = self.y + self.rStick
        elseif yv > 0.5 then
            self.digitalV = "b"
            y = self.y + self.h - self.rStick
        else
            self.digitalV = ""
            y = self.y + self.h / 2
        end
        
    elseif self.digital == "4" then-- 4 directions joystick:
        --ToDo
    end

    return x, y
end


function Joystick:move(direction)
    if (self.pressed or self.touchData) and self.stickPressed then
      --  local daX, daY = love.mouse.getPosition()
     --   daX = daX / gooi.sx
       -- daY = daY / gooi.sy
        if self.touchData then
         --   daX, daY = self.touchData.x, self.touchData.y
        end
        for bx,i in ipairs(love.touch.getTouches()) do
            local x,y = love.touch.getPosition(i)
            if self:overItAux(x,y) then
                daX,daY=x,y
            end
        end
        if self:butting() then
            local dX = self:theX() - daX - self.dx
            local dY = self:theY() - daY - self.dy
            local angle = (math.atan2(dY, dX) + math.rad(180));
            self.xStick = self.x + (self.r - self.rStick) * math.cos(angle) + self.r
            self.yStick = self.y + (self.r - self.rStick) * math.sin(angle) + self.r
            self.angle = angle
        else
            self.xStick, self.yStick = daX + self.dx, daY + self.dy --error()
        end
        if self.events.m then
            return self.events.m(self)
        end
    end
end
  
function Joystick:restore()
    if self.spring then
       self.xStick, self.yStick = self:theX(), self:theY()
    end
    self.stickPressed = false
    self.touchData = nil
    self.dx = 0
    self.dy = 0
end

function Joystick:noSpring()
    self.spring = false
    return self
end

function Joystick:butting()
    local hyp = 0
    local daX, daY
   -- local daX, daY = love.mouse.getPosition()
  --  daX = daX / gooi.sx
 --   daY = daY / gooi.sy
    if self.touchData then
        daX, daY = self.touchData.x, self.touchData.y
    end
    hyp = math.sqrt(
        math.pow(self:theX() - daX - self.dx, 2) +
        math.pow(self:theY() - daY - self.dy, 2))
    return hyp >= self.r - self.rStick
end
  
-- Get numbers with presicion of two decimals:
function Joystick:xValue()
    if self:onDeadZone() then return 0 end
    return gooi.round((self.xStick - self:theX()) / (self.r - self.rStick), 2)
end

function Joystick:yValue()
    if self:onDeadZone() then return 0 end
    return gooi.round((self.yStick - self:theY()) / (self.r - self.rStick), 2)
end
function Joystick:getX()
    if self:onDeadZone() then return 0 end
    return gooi.round((self.xStick - self:theX()) / (self.r - self.rStick), 2)
end

function Joystick:getY()
    if self:onDeadZone() then return 0 end
    return gooi.round((self.yStick - self:theY()) / (self.r - self.rStick), 2)
end

function Joystick:direction()
    if self.digital then
        return self.digitalV..self.digitalH
    else
        return ""
    end
end

function Joystick:overStick(x, y)
    local dx = (self.xStick - x)
    local dy = (self.yStick - y)
    return math.sqrt(math.pow(dx, 2) + math.pow(dy, 2)) < self.rStick * 1.1
end

function Joystick:onDeadZone()
    local dx, dy = self:theX() - self.xStick, self:theY() - self.yStick
    return math.sqrt(math.pow(dx, 2) + math.pow(dy, 2)) <= self.deadZone * (self.r - self.rStick)
end
  
function Joystick:theX() return (self.x) + (self.r) end
function Joystick:theY() return (self.y) + (self.r) end
  cwarn=null
function Joystick:pressed(xt, yt)-- assert(xt) log(xt..", "..yt)
    if self.prsed then return end
    local x = xt or love.mouse.getX()
    local y = yt or love.mouse.getY()
    x = x / self.sx
    y = y / self.sy
    self.prsd = true
    self:setTooltip(nil, true)-- remove tooltips when something is clicked
    if self.enabled and self.visible then
        if self:overIt(x, y) then
            self.touchData = {x=x,y=y}
            if self.anyP then
                self.stickPressed = true error()
                self.dx = 0
                self.dy = 0
            elseif self:overStick(x, y) then
                self.stickPressed = true
            self.ttouchData = {x=x,y=y}
                self.dx = self.xStick - x
                self.dy = self.yStick - y
            end
        end
    end
    if self.events.p then
        return self.events.p(self)
    end
end

function Joystick:released(id, xt, yt)
self.prsd=false
    warn(self:xValue()..", "..self:yValue())--.." , deadxone: "..self:onDeadZone())
    self:restore()
    if self.events.r then
        return self.events.r(self)
    end
end

function Joystick:update(dt)
    self:move()
end

 function Joystick:setTooltip(text, reset)
    self.tooltip = text or self.tooltip

    if reset then
      self.timerTooltip = 0
      self.showTooltip = false
    end

    return self
end

function Joystick:onPress(f)
    self.events.p = f
    return self
end

function Joystick:onRelease(f)
    self.events.r = f
    return self
end

function Joystick:onMoved(f)
    self.events.m = f
    return self
end

function Joystick:setBounds(x, y, w, h)
  local theX = x or self.x
  local theY = y or self.y
  local theW = w or self.w
  local theH = h or self.h

  self.x, self.y, self.w, self.h = theX, theY, theW, theH

  if self.type == "joystick" or self.type == "knob" then
    self.smallerSide = self.h
    if self.w < self.h then
      self.smallerSide = self.w
    end
    self.w, self.h = self.smallerSide, self.smallerSide
    self:rebuild()
  end

  return self
end

function Joystick:setBGImage(image)
  if type(image) == "string" then
    image = love.graphics.newImage(image)
  end

  self.bgImage = image
  return self
end

function Joystick:setOpaque(b)
  self.opaque = b
  return self
end 
function Joystick:overItAux(x,y)
  -- Scale:
--  local xm = love.mouse.getX() / self.sx
 -- local ym = love.mouse.getY() / self.sy

  if self.touch then
    xm, ym = self.touch.x, self.touch.y-- Already scaled.
  end
  -- Scale:
  if x and y then
    xm, ym = x, y
  end

  local radiusCorner = self.style.radius

  local theX = self.x
  local theY = self.y
  local theW = self.w
  local theH = self.h

  -- Check if one of the "two" rectangles is on the mouse/finger:
  local b = not (
        xm < theX or
        ym < theY + radiusCorner or
        xm > theX + theW or
        ym > theY + theH - radiusCorner
        ) or not (
            xm < theX + radiusCorner or
            ym < theY or
            xm > theX + theW - radiusCorner or
            ym > theY + theH
        )

    -- Check if mouse/finger is over one of the 4 "circles":

    local x1, x2, y1, y2 =
        theX + radiusCorner,
        theX + theW - radiusCorner,
        theY + radiusCorner,
        theY + theH - radiusCorner

    local hyp1 = math.sqrt(math.pow(xm - x1, 2) + math.pow(ym - y1, 2))
    local hyp2 = math.sqrt(math.pow(xm - x2, 2) + math.pow(ym - y1, 2))
    local hyp3 = math.sqrt(math.pow(xm - x1, 2) + math.pow(ym - y2, 2))
    local hyp4 = math.sqrt(math.pow(xm - x2, 2) + math.pow(ym - y2, 2))

    return (hyp1 < radiusCorner or
        hyp2 < radiusCorner or
        hyp3 < radiusCorner or
        hyp4 < radiusCorner or b), index, xm, ym
end

function Joystick:overIt(x, y)-- x and y if it's the first time pressed (no touch defined yet).
    if self.type == "panel" or self.type == "label" then
        return false
    end

    -- Not applicable in this case:
    if not (self.enabled or self.visible) then return false end
  --  log
    if self.noFlag or self.okFlag or self.yesFlag then
        return self:overItAux(x, y)
    else
        if nil then--????gooi.showingDialog then
            return false
        else
            return self:overItAux(x, y)
        end
    end
end

return Joystick